@@ -275,6 +275,8 @@ amap_struct_triggers *read_file_triggers(char *extension, char *filename, amap_s
         memset(trigger, 0, sizeof(amap_struct_triggers));
       }
       line[strlen(line) - 1] = 0;
+      if (line[strlen(line) - 1] == '\r')
+        line[strlen(line) - 1] = 0;
       strcpy(orig_line, line);
       t_uid = line;
       ports = amap_index(t_uid, ':');
@@ -440,6 +442,8 @@ amap_struct_responses *read_file_responses(char *extension, char *filename, amap
         memset(response, 0, sizeof(amap_struct_responses));
       }
       line[strlen(line) - 1] = 0;
+      if (line[strlen(line) - 1] == '\r')
+        line[strlen(line) - 1] = 0;
       strcpy(orig_line, line);
       t_uid = line;
       triggerptr = amap_index(t_uid, ':');
@@ -534,6 +538,8 @@ amap_struct_ports *amap_add_port_string(amap_struct_ports *port_tmp, char *port,
   int pfrom, pto;
 
   if ((ptr = index(port, '-')) == NULL) {
+    if (atoi(port) < 1 || atoi(port) > 65535)
+      amap_error("ports to be scanned must be between 1 and 65535 inclusive: %s", port);
     port_tmp->port = atoi(port);
     port_tmp->ip_prot = cmd_proto;
   } else {
@@ -541,7 +547,7 @@ amap_struct_ports *amap_add_port_string(amap_struct_ports *port_tmp, char *port,
     ptr++;
     pfrom = atoi(port);
     pto = atoi(ptr);
-    if (pto < 1 || pfrom < 1 || pto < pfrom)
+    if (pto < 1 || pfrom < 1 || pto < pfrom || pto > 65535 || pfrom > 65535)
       amap_error("range definition is invalid: %s-%s",port, ptr);
     for ( ;  pfrom <= pto; pfrom++) {
       port_tmp->port = (unsigned short int) pfrom;
@@ -574,6 +580,7 @@ amap_struct_targets *read_file_nmap(char *filename, amap_struct_options *opt) {
   char *proto;
   char *ptr;
   int ip_prot;
+  int count = 0;
 
   f = amap_open_file(filename, "nmap", "", opt->verbose);
   if (((amap_struct_targets*) targets = (amap_struct_targets*) target = (amap_struct_targets*) malloc(sizeof(amap_struct_targets))) == NULL)
@@ -587,7 +594,7 @@ amap_struct_targets *read_file_nmap(char *filename, amap_struct_options *opt) {
     if (line[strlen(line) - 1] != '\n')
       amap_error("line in nmap file is too long or not terminating with \\n: %s", line);
     if ((line[0] == 'H') && (index(line, ' ') != NULL) && ((portinfo = strstr(line, "/open/")) != NULL)) { // just care for data lines
-      if (targets != target) {
+      if (count != 0) {
         if (((amap_struct_targets*) target->next = (amap_struct_targets*) malloc(sizeof(amap_struct_targets))) == NULL)
           amap_error("malloc failed");
         target = (amap_struct_targets*) target->next;
@@ -597,6 +604,8 @@ amap_struct_targets *read_file_nmap(char *filename, amap_struct_options *opt) {
         memset(port_tmp, 0, sizeof(amap_struct_ports));
       }
       line[strlen(line) - 1] = 0;
+      if (line[strlen(line) - 1] == '\r')
+        line[strlen(line) - 1] = 0;
       strcpy(orig_line, line);
       ip = index(line, ' ');
       ip++;
@@ -649,6 +658,7 @@ amap_struct_targets *read_file_nmap(char *filename, amap_struct_options *opt) {
         if (opt->verbose > 1)
           printf("%d/%s ", port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
       }
+      count++;
       if (opt->verbose > 1)
         printf("\n");
     }
@@ -659,6 +669,8 @@ amap_struct_targets *read_file_nmap(char *filename, amap_struct_options *opt) {
     free(targets);
     targets = NULL;
   }
+  if (opt->verbose)
+    printf("done\n");
   return targets;
 }
 
@@ -978,15 +990,19 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
             response = (amap_struct_responses*) response->next;
           }
           // if !found then now response matched the received data, report this
-          if (found < 1 && coms[i].port->unknown_response == NULL && coms[i].port->ids == NULL) {
+          if (found < 1 && (opt->dump_all || (coms[i].port->unknown_response == NULL && coms[i].port->ids == NULL))) {
+            if (opt->verbose)
+              snprintf(info, sizeof(info), "(by trigger %s) ", coms[i].trigger->id);
+            else
+              info[0] = 0;
             if (opt->logfile != NULL && !opt->machine_readable) {
-              fprintf(opt->logfile, "Unrecognized response from %s:%d/%s%sreceived.\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ");
+              fprintf(opt->logfile, "Unrecognized response from %s:%d/%s%s%sreceived.\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
               if (opt->dump_unidentified) {
                 fprintf(opt->logfile, "Please send output + name of the application to %s:\n", AMAP_EMAIL);
                 amap_dump_string(opt->logfile, coms[i].response, coms[i].response_length, 16);
               }
             }
-            printf("Unrecognized response from %s:%d/%s%sreceived.\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ");
+            printf("Unrecognized response from %s:%d/%s%s%sreceived.\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info);
             if (opt->dump_unidentified) {
               printf("Please send this output and the name of the application to %s:\n", AMAP_EMAIL);
               amap_dump_string(stdout, coms[i].response, coms[i].response_length, 16);
@@ -1910,12 +1926,12 @@ int main(int argc, char *argv[]) {
             ptr++;
             pfrom = atoi(argv[i]);
             pto = atoi(ptr);
-            if (pto < 1 || pfrom < 1 || pto < pfrom)
-              amap_error("range definition is invalid: %s-%s",argv[i], ptr);
           } else {
             pto = atoi(argv[i]);
             pfrom = pto;
           }
+          if (pto < 1 || pfrom < 1 || pto < pfrom)
+            amap_error("range definition is invalid: %s-%s",argv[i], ptr);
           for ( ; pfrom <= pto; pfrom++) {
             port_tmp = target_tmp->ports;
             while (((port_tmp->ip_prot != cmd_proto) || (port_tmp->port != (unsigned short int) pfrom)) && port_tmp->next != NULL)
@@ -1971,7 +1987,7 @@ int main(int argc, char *argv[]) {
     if (opt.machine_readable)
       fprintf(opt.logfile, "# IP_ADDRESS:PORT:PROTOCOL:PORT_STATUS:SSL:IDENTIFICATION:PRINTABLE_BANNER:FULL_BANNER\n");
   }
-  
+
   // STARTING SCAN //
   if (opt.verbose)
     printf("\n");
